@@ -21,10 +21,7 @@
 #
 # SPDX-License-Identifier: curl
 #
-"""Module for extracting test data from the test data folder and other utils"""
-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+"""Module for extracting test data from the test data folder and other utils."""
 
 import logging
 import os
@@ -75,8 +72,8 @@ class TestData(object):
 
         log.debug("Parsing file %s", filename)
 
-        with open(filename, "rb") as f:
-            contents = f.read().decode("utf-8")
+        with open(filename, "r", encoding='us-ascii') as f:
+            contents = f.read()
 
         m = REPLY_DATA.search(contents)
         if not m:
@@ -88,5 +85,5 @@ class TestData(object):
 
 if __name__ == '__main__':
     td = TestData("./data")
-    data = td.get_test_data(1)
+    data = td.get_test_data(1451)
     print(data)
